SystemOrganization addCategory: #'Ocompletion-ECModel'!

InstructionClient subclass: #ECVarTypeGuesser
	instanceVariableNames: 'types receiverClass variableName found currentMethod hasSend contextCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECVarTypeGuesser commentStamp: '<historical>' prior: 0!
I'm an InstructionClient that tries to guess the type of a given instance variable name of a class. !

ECVarTypeGuesser subclass: #ECClassVarTypeGuesser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECClassVarTypeGuesser methodsFor: 'public' stamp: 'lr 7/4/2009 10:42'!
methodRefs
	| theClass classVarAssoc |
	theClass := receiverClass classThatDefinesClassVariable: variableName.
	classVarAssoc := theClass classPool associationAt: variableName asSymbol.
	classVarAssoc value 
		ifNil: [ ^ SystemNavigation new allCallsOn: classVarAssoc ]
		ifNotNil: [ ^ classVarAssoc value class ]! !

!ECClassVarTypeGuesser methodsFor: 'instruction decoding' stamp: 'EduardoEscobar 1/10/2012 21:59'!
popIntoLiteralVariable: anAssociation 
	"Remove Top Of Stack And Store Into Literal Variable bytecode."
	anAssociation key == variableName asSymbol 
		ifTrue: [ found := true ]
		ifFalse: [ self reset ]! !

!ECClassVarTypeGuesser methodsFor: 'instruction decoding' stamp: 'EduardoEscobar 1/10/2012 22:00'!
popIntoReceiverVariable: offset 
	"Remove Top Of Stack And Store Into Instance Variable bytecode."
	self reset! !

!ECClassVarTypeGuesser methodsFor: 'instruction decoding' stamp: 'EduardoEscobar 1/10/2012 22:00'!
send: selector super: supered numArgs: numberArguments 
	"Send Message With Selector, selector, bytecode. The argument, supered, indicates whether the receiver of the message is specified with 'super' in the source method. The arguments of the message are found 
in the top numArguments locations on the stack and the receiver just below them."
! !

ECVarTypeGuesser subclass: #ECInstVarTypeGuesser
	instanceVariableNames: 'varIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECInstVarTypeGuesser commentStamp: '<historical>' prior: 0!
I'm a simple InstructionClient that tries to guess the type of a given instance variable name of a class. !

!ECInstVarTypeGuesser methodsFor: 'public' stamp: 'lr 9/26/2010 17:44'!
methodRefs
	| theClass selectors |
	theClass := receiverClass classThatDefinesInstanceVariable: variableName.
	theClass ifNil: [ ^ nil ].
	selectors := theClass whichSelectorsStoreInto: variableName.
	^ selectors collect: [ :each | MethodReference new setStandardClass: theClass methodSymbol: each ]! !

!ECInstVarTypeGuesser methodsFor: 'bytecode decoding' stamp: 'EduardoEscobar 1/10/2012 22:19'!
popIntoReceiverVariable: offset 
	"Remove Top Of Stack And Store Into Instance Variable bytecode."
	varIndex = offset 
		ifTrue: [ found := true ]
		ifFalse: [ self reset ]! !

!ECInstVarTypeGuesser methodsFor: 'bytecode decoding' stamp: 'lr 7/4/2009 10:42'!
prepare: aCompiledMethod 
	| theClass |
	super prepare: aCompiledMethod.
	theClass := aCompiledMethod actualClass.
	varIndex := (theClass allInstVarNames indexOf: variableName) - 1! !

!ECVarTypeGuesser class methodsFor: 'as yet unclassified' stamp: 'ul 12/9/2010 23:08'!
getClassFromTypeSuggestingName: aString 
	| firstUppercaseLetter className aStream aClass |
	aStream := WriteStream on: String new.
	aStream nextPut: aString first asUppercase.
	aStream
		nextPutAll: (aString copyFrom: 2 to: aString size).
	className := aStream contents.
	aClass := Smalltalk
				at: className asSymbol
				ifAbsent: [].
	(aClass isKindOf: Class)
		ifTrue: [^ aClass].
	firstUppercaseLetter := aString findFirst: [ :each | each isUppercase ].
	className := firstUppercaseLetter > 1
				ifTrue: [aString copyFrom: firstUppercaseLetter to: aString size]
				ifFalse: [^ nil].
	aClass := Smalltalk
				at: className asSymbol
				ifAbsent: [].
	(aClass isKindOf: Class)
		ifTrue: [^ aClass].
	^ nil! !

!ECVarTypeGuesser class methodsFor: 'as yet unclassified' stamp: 'bar 12/8/2004 13:47'!
variableName: aString class: aClass 
	^self variableName: aString source: nil class: aClass ! !

!ECVarTypeGuesser class methodsFor: 'as yet unclassified' stamp: 'lr 7/4/2009 10:42'!
variableName: aString source: sourceString class: aClass 
	| newInstance |
	newInstance := self basicNew initialize.
	newInstance 
		setVariableName: aString
		source: sourceString
		class: aClass.
	^ newInstance! !

!ECVarTypeGuesser methodsFor: 'bytecode decoding' stamp: 'EduardoEscobar 1/10/2012 22:24'!
blockReturnTop
	"Return Top Of Stack bytecode."
	contextCount := contextCount - 1! !

!ECVarTypeGuesser methodsFor: 'bytecode decoding' stamp: 'lr 7/4/2009 10:42'!
computeVarType
	| info tempNames name |
	types ifEmpty: [ ^ nil ].
	info := types first.
	(hasSend and: [ info isDefinedByMessageSend not ]) ifTrue: 
		[ info type: nil.
		^ info ].
	info isDefinedByTemporary not ifTrue: [ ^ info ].
	tempNames := (receiverClass compilerClass new 
		parse: currentMethod getSourceFromFile asString
		in: receiverClass
		notifying: nil) tempNames.
	name := tempNames at: info temporaryOffset + 1.
	info type: (self class getClassFromTypeSuggestingName: name).
	^ info! !

!ECVarTypeGuesser methodsFor: 'instance creation' stamp: 'EduardoEscobar 1/10/2012 22:24'!
initialize
	"Subclasses should redefine this method to perform initializations on instance creation."
	super initialize.
	types := OrderedCollection new.
	hasSend := false.
	contextCount := 0! !

!ECVarTypeGuesser methodsFor: 'bytecode decoding' stamp: 'bar 12/1/2004 21:47'!
interpretNextInstructionUsing: aScanner 
	found := false.
	aScanner interpretNextInstructionFor: self.
	^found
! !

!ECVarTypeGuesser methodsFor: 'public' stamp: 'bar 3/20/2006 15:59'!
methodRefs
	^Array new! !

!ECVarTypeGuesser methodsFor: 'public' stamp: 'ul 9/29/2010 22:00'!
perform
	| minPriorityInfo  methodRefs |
	variableName isEmpty
		ifTrue: [^ nil].
	methodRefs := self methodRefs.
	methodRefs
		ifNil: [^ nil]. 
	methodRefs isBehavior
		ifTrue: [^ methodRefs].
	contextCount := 0.
	minPriorityInfo := nil.
	methodRefs do: [:each | 
		| infosOfMethod |
		self prepare: each.
		infosOfMethod := self typeOfVarIn: each compiledMethod.
		(infosOfMethod detectMin: #priority) ifNotNil: [ :possibleMinPrioriotyInfo |
			minPriorityInfo 
				ifNil: [ minPriorityInfo := possibleMinPrioriotyInfo ]
				ifNotNil: [ 
					minPriorityInfo priority <= possibleMinPrioriotyInfo priority
						ifFalse: [ minPriorityInfo := possibleMinPrioriotyInfo ] ] ] ].
	^minPriorityInfo ifNotNil: [ minPriorityInfo type ]! !

!ECVarTypeGuesser methodsFor: 'bytecode decoding' stamp: 'bar 8/11/2005 13:03'!
popIntoLiteralVariable: anAssociation 
	"Remove Top Of Stack And Store Into Literal Variable bytecode."
	self reset! !

!ECVarTypeGuesser methodsFor: 'bytecode decoding' stamp: 'bar 12/13/2004 17:08'!
popIntoTemporaryVariable: offset 
	"Remove Top Of Stack And Store Into Temporary Variable bytecode."
	self reset
! !

!ECVarTypeGuesser methodsFor: 'bytecode decoding' stamp: 'bar 8/11/2005 13:19'!
prepare: aCompiledMethod ! !

!ECVarTypeGuesser methodsFor: 'bytecode decoding' stamp: 'lr 7/4/2009 10:42'!
pushActiveContext
	"Push Active Context On Top Of Its Own Stack bytecode."
	contextCount := contextCount + 1! !

!ECVarTypeGuesser methodsFor: 'bytecode decoding' stamp: 'lr 7/4/2009 10:42'!
pushConstant: value 
	"Push Constant, value, on Top Of Stack bytecode."
	| info |
	contextCount > 0 ifTrue: [ ^ self ].
	value ifNotNil: 
		[ info := ECTypeInfo definedByLiteral: value class.
		types add: info ]! !

!ECVarTypeGuesser methodsFor: 'bytecode decoding' stamp: 'bar 12/13/2004 17:15'!
pushLiteralVariable: anAssociation 
	"Push Contents Of anAssociation On Top Of Stack bytecode."
	| info |
	contextCount > 0 ifTrue:[^self].

	info := ECTypeInfo definedByMessageSend: anAssociation value.
	types add: info! !

!ECVarTypeGuesser methodsFor: 'bytecode decoding' stamp: 'EduardoEscobar 1/10/2012 22:25'!
pushTemporaryVariable: offset 
	"Push Contents Of Temporary Variable Whose Index Is the argument, offset, On Top Of Stack bytecode."
	| info |
		contextCount > 0 ifTrue:[^self].

	info := ECTypeInfo definedByTemporaryVar: offset.
	types add: info! !

!ECVarTypeGuesser methodsFor: 'private' stamp: 'bar 12/13/2004 17:14'!
reset
	contextCount > 0 ifTrue:[^self].
	types reset.
	hasSend := false.! !

!ECVarTypeGuesser methodsFor: 'bytecode decoding' stamp: 'bar 12/13/2004 17:15'!
send: selector super: supered numArgs: numberArguments 
	"Send Message With Selector, selector, bytecode. The argument,  
	supered, indicates whether the receiver of the message is specified with 
	'super' in the source method. The arguments of the message are found  
	in  
	the top numArguments locations on the stack and the receiver just  
	below them."
	contextCount > 0 ifTrue:[^self].

	hasSend := true! !

!ECVarTypeGuesser methodsFor: 'instance creation' stamp: 'bar 12/13/2004 13:55'!
setVariableName: aString source: aSourceString class: aClass 
	variableName := aString.
	receiverClass := aClass! !

!ECVarTypeGuesser methodsFor: 'bytecode decoding' stamp: 'lr 7/4/2009 10:42'!
typeOfVarIn: aMethod 
	"Answer whether the receiver references an instance variable."
	| scanner end type infos |
	scanner := InstructionStream on: aMethod.
	end := scanner method endPC.
	currentMethod := aMethod.
	infos := OrderedCollection new.
	[ scanner pc <= end ] whileTrue: 
		[ (self interpretNextInstructionUsing: scanner) ifTrue: 
			[ type := self computeVarType.
			type ifNotNil: [ infos add: type ] ] ].
	^ infos! !

Object subclass: #ECContext
	instanceVariableNames: 'source position theClass ranges completionIndex recurseCount receiverClass completionToken model controller variables selectors typeInfos'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECContext commentStamp: '<historical>' prior: 0!
A completion is started by the ECController. The controller creates me to compute the context of the completion. The most important information about the context are the receiverClass and the completionToken. I create a ECModel or subclass when requested by the 'model' method.

I use SHParser and SHRange to parse the text input.!

!ECContext class methodsFor: 'instance creation' stamp: 'bar 4/11/2005 11:23'!
controller: aECController class: aClass source: aString position: anInteger 
	^ self basicNew initialize
		setController: aECController
		class: aClass
		source: aString
		position: anInteger! !

!ECContext methodsFor: 'private-temporaries' stamp: 'lr 7/4/2009 10:42'!
blockTemporaries
	| blocks range vars |
	blocks := OrderedCollection new.
	1 
		to: completionIndex - 1
		do: 
			[ :index | 
			range := ranges at: index.
			self 
				handleBlockStack: blocks
				with: range.
			range isBlockTemporary ifTrue: 
				[ vars := blocks last.
				vars add: range ] ].
	^ self convertBlocksToVariables: blocks! !

!ECContext methodsFor: 'private-compute-index' stamp: 'RomainRobbes 11/2/2009 07:56'!
checkForZeroPosition
	^ (position = 0
			or: [ranges isEmpty])
		and: [self insertEmptyRangeAt: 1 start: 0 end: 1.
			true]! !

!ECContext methodsFor: 'private-receiver-guessing' stamp: 'bar 1/9/2005 14:55'!
checkImpossibleReceiver
	^ self isSelectorsAndVariables
		ifTrue: [self configureSelectorsAndVariables]
		ifFalse: [self isVariablesOnly
				ifTrue: [self configureVariablesOnly]
				ifFalse: [self isSelectorsOnly
						and: [self configureSelectorsOnly]]]. ! !

!ECContext methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
completionToken
	completionToken ifNil: 
		[ | range |
		range := ranges at: completionIndex.
		completionToken := self 
			sourceOf: range
			stopAt: position.
		completionToken := completionToken wordBefore: completionToken size ].
	^ completionToken! !

!ECContext methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
compute
	completionIndex := self computeIndexOfPosition.
	receiverClass := self computeReceiverClass! !

!ECContext methodsFor: 'private-compute-index' stamp: 'lr 7/4/2009 10:42'!
computeIndexOfPosition
	| current |
	self checkForZeroPosition ifTrue: [ ^ 1 ].
	1 
		to: ranges size
		do: 
			[ :index | 
			current := ranges at: index.
			(current includesPosition: position) 
				ifTrue: [ ^ index ]
				ifFalse: 
					[ current end > position ifTrue: [ ^ self createEmptyRangeForGapAt: index ] ] ].
	^ self createEmptyRangeAtTail! !

!ECContext methodsFor: 'private' stamp: 'bar 1/9/2005 11:35'!
computeReceiverClass
	| previous found |
	recurseCount := 0.
	completionIndex = 1 ifTrue: [^nil].
	found := self checkImpossibleReceiver.
	found ifTrue: [^nil].
	previous := ranges at: completionIndex - 1.
	previous type = #cascadeSeparator 
		ifTrue: [^self guessCascadeReceiver: completionIndex - 1].
	(previous type = #unary 
		and: [(self sourceOf: previous) = 'new' and: [completionIndex > 2]]) 
			ifTrue: 
				[previous := ranges at: completionIndex - 2.
				previous type = #globalVar 
					ifTrue: [^self guessTypeOf: completionIndex - 2]
					ifFalse: 
						[self configureSelectorsOnly.
						^nil]].
	^self guessTypeOf: completionIndex - 1! !

!ECContext methodsFor: 'private-configure' stamp: 'bar 1/9/2005 11:35'!
configureSelectorsAndVariables
	variables := true.
	selectors := true.
	^true! !

!ECContext methodsFor: 'private-configure' stamp: 'bar 1/9/2005 11:35'!
configureSelectorsOnly
	variables := false.
	selectors := true.
	^true! !

!ECContext methodsFor: 'private-configure' stamp: 'bar 1/9/2005 11:34'!
configureVariablesOnly
	variables := true.
	selectors := false.
	^true! !

!ECContext methodsFor: 'private-temporaries' stamp: 'lr 7/4/2009 10:42'!
convertBlocksToVariables: anOrderedCollection 
	| result blockStack |
	blockStack := anOrderedCollection.
	result := OrderedCollection new.
	blockStack do: [ :each | result addAll: each ].
	^ result! !

!ECContext methodsFor: 'private-compute-index' stamp: 'lr 7/4/2009 10:42'!
createEmptyRangeAtTail
	| previous |
	previous := ranges last.
	ranges add: (SHRange 
			start: previous end + 1
			end: source size
			type: #empty).
	^ ranges size! !

!ECContext methodsFor: 'private-compute-index' stamp: 'RomainRobbes 11/2/2009 07:57'!
createEmptyRangeForGapAt: index 
	| current previous |
	index = 1 ifTrue: [self insertEmptyRangeAt: 1 start: 0 end: 1. ^ index].
	current := ranges at: index.
	previous := ranges at: index - 1.
	self 
		insertEmptyRangeAt: index
		start: previous end + 1
		end: current start - 1.
	^ index! !

!ECContext methodsFor: 'accessing' stamp: 'dr 10/30/2008 17:15'!
createModel
	| modelClass |
	self receiverClass
		ifNotNil: [^ ECTypedModel class: receiverClass].
	(controller workspace isNil
		and: [theClass notNil])
		ifTrue: [completionIndex = 1
				ifTrue: [^ ECOverrideModel class: theClass]].
			
	modelClass := ECUntypedModel.		
	(controller model respondsTo: #modelClass) ifTrue: [controller model modelClass ifNotNilDo: [:class | modelClass :=  class]].		
	^ modelClass
		class: theClass
		temporaries: self temporaries
		additionals: controller additionals
		variables: variables
		selectors: selectors! !

!ECContext methodsFor: 'private' stamp: 'bar 1/6/2005 14:35'!
createRanges
	| parser |
	parser := SHParserST80 new.
	ranges := parser 
				rangesIn: source
				classOrMetaClass: theClass
				workspace: controller workspace
				environment: nil.
	ranges := ranges select: [:each | each type ~= #comment]! !

!ECContext methodsFor: 'private-roel-typer' stamp: 'lr 3/26/2010 15:04'!
findCommonSuperclass: aCollection
	| current |
	aCollection isEmpty
		ifTrue: [ ^ nil ].
	current := aCollection first.
	aCollection do: [ :class |
		[ class includesBehavior: current ]
			whileFalse: [ current := current superclass ] ].
	^ current! !

!ECContext methodsFor: 'private' stamp: 'bar 1/9/2005 14:51'!
findSourceRangeFor: aNumber 
	aNumber
		to: ranges size
		by: 2
		do: [:index | 
			index + 3 > ranges size
				ifTrue: [^ nil].
			(ranges at: index + 1) isAssignment
				ifTrue: [(ranges at: index + 3) type = #statementSeparator
						ifTrue: [^ index + 2]
						ifFalse: [(ranges at: index + 2) type = #globalVar
								ifTrue: [^ index + 2]
								ifFalse: [(ranges at: index + 1) isAssignment 
										ifFalse: [^ nil]]]]]! !

!ECContext methodsFor: 'private-type-guessing' stamp: 'lr 7/4/2009 10:42'!
guessArgument: aSHRange 
	| name |
	name := self sourceOf: aSHRange.
	(name = 'html' and: [ (Smalltalk 
		at: #WARenderCanvas
		ifAbsent: [  ]) notNil ]) ifTrue: [ ^ Smalltalk at: #WARenderCanvas ].
	^ ECInstVarTypeGuesser getClassFromTypeSuggestingName: name! !

!ECContext methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
guessCascadeReceiver: aNumber 
	| type |
	aNumber 
		to: 1
		by: -1
		do: 
			[ :index | 
			type := (ranges at: index) type.
			((#(#statementSeparator #assignment #ansiAssignment ) includes: type) or: [ (type beginsWith: 'pattern') or: [ type beginsWith: 'methodTemp' ] ]) ifTrue: [ ^ self guessTypeOf: index + 1 ] ].
	^ nil! !

!ECContext methodsFor: 'private' stamp: 'damiencassou 7/18/2009 08:05'!
guessClassVarClass: aSHRange 
	| aClass name |
	name := self sourceOf: aSHRange.
	aClass := controller guessTypeFor: name.
	aClass
		ifNotNil: [^ aClass].
	^ (ECClassVarTypeGuesser variableName: name class: theClass theMetaClass) perform! !

!ECContext methodsFor: 'private-type-guessing' stamp: 'lr 7/4/2009 10:42'!
guessGlobal: aNumber 
	| aClass |
	aClass := Smalltalk 
		at: (self sourceOf: (ranges at: aNumber)) asSymbol
		ifAbsent: [ ^ nil ].
	aClass isBehavior ifFalse: [ ^ nil ].
	aNumber = (completionIndex - 1) ifTrue: [ ^ aClass class ].
	(ranges size >= (aNumber + 1) and: [ (ranges at: aNumber + 1) type = #statementSeparator ]) ifTrue: [ ^ aClass class ].
	^ aClass! !

!ECContext methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
guessInstVarClass: aSHRange 
	| aClass name |
	name := self sourceOf: aSHRange.
	aClass := controller guessTypeFor: name.
	aClass ifNotNil: [ ^ aClass ].
	aClass := self 
		guessWithRoelTyper: name
		class: theClass.
	aClass ifNotNil: [ ^ aClass ].
	^ (ECInstVarTypeGuesser 
		variableName: name
		class: theClass) perform! !

!ECContext methodsFor: 'private' stamp: 'bar 3/1/2006 13:52'!
guessTempVarClass: aSHRange 
	^self guessTempVarClass: (self sourceOf: aSHRange) type: aSHRange type.
! !

!ECContext methodsFor: 'private' stamp: 'ul 1/28/2011 00:29'!
guessTempVarClass: aString type: aSymbol 
	| current type varName varType sourceIndex aClass |
	aClass := controller guessTypeFor: aString.
	aClass ifNotNil: [ ^ aClass ].
	varName := aString.
	varType := aSymbol.
	completionIndex
		to: 1
		by: -1
		do: 
			[ :index | 
			current := ranges at: index.
			(current type = varType and: [ (self sourceOf: current) = varName and: [ index + 3 <= ranges size ] ]) ifTrue: 
				[ (sourceIndex := self findSourceRangeFor: index) notNil ifTrue: 
					[ type := self guessTypeOf: sourceIndex.
					type ifNotNil: [ ^ type ] ] ] ].
	^ nil! !

!ECContext methodsFor: 'private-type-guessing' stamp: 'ul 1/28/2011 01:04'!
guessTypeOf: aNumber

	| range |
	self configureSelectorsOnly.
	recurseCount > 10 ifTrue: [ ^ nil ].
	recurseCount := recurseCount + 1.
	range := ranges at: aNumber.
	range isSelf ifTrue: [ ^theClass ].
	range isSuper ifTrue: [
		^theClass ifNotNil: [
			(theClass isKindOf: Class) ifTrue: [ "It may be a Trait or something else."
				theClass superclass ] ] ].
	range asType ifNotNil: [ :type | "Constant"
		type == Number ifFalse: [ ^type ].
		^(SqNumberParser 
			parse: (
				source
					copyFrom: range start
					to: range end)
			onError: [ ^Number ]) class ].
	range isArgument ifTrue: [ ^self guessArgument: range ].
	range isTemporaryVariable ifTrue: [ ^self guessTempVarClass: range ].
	range isInstanceVariable ifTrue: [ ^self guessInstVarClass: range ].
	range isClassVariable ifTrue: [ ^self guessClassVarClass: range ].
	range isGlobal ifTrue: [ ^self guessGlobal: aNumber ].
	^nil! !

!ECContext methodsFor: 'private-roel-typer' stamp: 'lr 3/26/2010 15:16'!
guessWithRoelTyper: aString class: aClass 
	| typeCollector typeClass typeInfo types type |
	typeCollector := Smalltalk classNamed: #TypeCollector.
	typeCollector ifNil: [ ^ nil ].
	typeClass := aClass whichClassDefinesInstVar: aString.
	typeClass ifNil: [ ^ nil ].
	typeInfo := (typeCollector typeInstvarsOfClass: typeClass)
		at: aString asSymbol
		ifAbsent: [ ^ nil ].
	types := typeInfo types size <= 2
		ifTrue: [ typeInfo types ]
		ifFalse: [ typeInfo types intersection: typeInfo assignments ].
	type := types isEmpty
		ifFalse: [ self findCommonSuperclass: types ].
	^ type == Object 
		ifFalse: [ type ]! !

!ECContext methodsFor: 'private-temporaries' stamp: 'lr 7/4/2009 10:42'!
handleBlockStack: aCollection with: aSHRange 
	| range blockStack |
	range := aSHRange.
	blockStack := aCollection.
	range isBlockStart 
		ifTrue: [ blockStack add: OrderedCollection new ]
		ifFalse: [ range isBlockEnd ifTrue: [ blockStack removeLast ] ]! !

!ECContext methodsFor: 'initialize-release' stamp: 'EduardoEscobar 1/10/2012 22:04'!
initialize
	"Subclasses should redefine this method to perform initializations on instance creation"
	super initialize.
	source := String new.
	position := 0.
	recurseCount := 0.
	variables := true.
	selectors := true.
	ranges := OrderedCollection new.
	completionIndex := 0.
	completionToken := nil! !

!ECContext methodsFor: 'private-compute-index' stamp: 'bar 1/6/2005 22:04'!
insertEmptyRangeAt: index start: start end: end 
	ranges
		add: (SHRange
				start: start
				end: end
				type: #empty)
		beforeIndex: index! !

!ECContext methodsFor: 'private-receiver-guessing' stamp: 'bar 1/9/2005 11:33'!
isSelectorsAndVariables
	| current |
	current := ranges at: completionIndex.
	^current isUnfinished! !

!ECContext methodsFor: 'private-receiver-guessing' stamp: 'bar 1/9/2005 11:33'!
isSelectorsOnly
	| previous |
	previous := ranges at: completionIndex - 1.
	^previous isOpening! !

!ECContext methodsFor: 'private-receiver-guessing' stamp: 'lr 7/4/2009 10:42'!
isVariablesOnly
	| current previous |
	current := ranges at: completionIndex.
	^ current isVariablesOnly or: 
		[ current isOpening or: 
			[ previous := ranges at: completionIndex - 1.
			previous isOpening or: 
				[ previous isSeparator or: 
					[ previous isKeyword or: [ previous isAssignment or: [ previous isBinary ] ] ] ] ] ]! !

!ECContext methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
model
	model isNil ifTrue: [ model := self createModel ].
	^ model! !

!ECContext methodsFor: 'action' stamp: 'lr 7/4/2009 10:42'!
narrowWith: aString 
	completionToken := aString.
	model ifNotNil: [ model narrowWith: aString ]! !

!ECContext methodsFor: 'private' stamp: 'bar 12/13/2004 15:32'!
receiverClass
	^ receiverClass! !

!ECContext methodsFor: 'initialize-release' stamp: 'lr 7/4/2009 10:42'!
setController: aECController class: aClass source: aString position: anInteger 
	controller := aECController.
	theClass := aClass.
	source := aString.
	position := anInteger.
	self createRanges.
	self compute! !

!ECContext methodsFor: 'private' stamp: 'bar 3/1/2006 13:25'!
sourceOf: aSHRange 
	^aSHRange isString
		ifTrue: [aSHRange]
		ifFalse: [self sourceOf: aSHRange stopAt: aSHRange end]! !

!ECContext methodsFor: 'private' stamp: 'bar 12/18/2004 23:55'!
sourceOf: aSHRange stopAt: aNumber
	^ aSHRange type = #empty
		ifTrue: [String new]
		ifFalse: [source copyFrom: aSHRange start to: aNumber]! !

!ECContext methodsFor: 'action' stamp: 'bar 1/9/2005 11:35'!
switchToUntyped
	receiverClass := nil.
	self configureSelectorsAndVariables.
	model := self createModel! !

!ECContext methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
temporaries
	| tempRanges |
	tempRanges := ranges select: [ :each | #(#patternTempVar #patternArg ) includes: each type ].
	tempRanges addAll: self blockTemporaries.
	^ tempRanges collect: 
		[ :each | 
		ECLocalEntry 
			contents: (self sourceOf: each)
			type: each type ]! !

!ECContext methodsFor: 'accessing' stamp: 'bar 2/26/2006 09:04'!
theClass
	^theClass! !

!ECContext methodsFor: 'private-roel-typer' stamp: 'lr 7/4/2009 10:42'!
typeInfosFor: aClass 
	| roelTyper |
	(roelTyper := Smalltalk 
		at: #TypeCollector
		ifAbsent: [  ]) ifNil: [ ^ nil ].
	typeInfos ifNil: [ typeInfos := roelTyper typeInstvarsOfClass: aClass ].
	^ typeInfos! !

Object subclass: #ECEntry
	instanceVariableNames: 'contents type description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECEntry commentStamp: 'bar 10/5/2005 23:31' prior: 0!
I represent a completion entry that is management by a ECModel and shown in the ECMenuMorph as a menu entry.!

ECEntry subclass: #ECClassVarEntry
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECClassVarEntry methodsFor: 'as yet unclassified' stamp: 'bar 3/1/2006 15:27'!
guessTypeWith: anECContext 
	^ anECContext guessClassVarClass: contents! !

!ECClassVarEntry methodsFor: 'accessing' stamp: 'bar 3/1/2006 15:40'!
label
	^ 'class variable'! !

!ECEntry class methodsFor: 'as yet unclassified' stamp: 'bar 8/13/2005 18:55'!
contents: aString type: aSymbol
	^ self new setContents: aString type: aSymbol! !

!ECEntry methodsFor: 'comparing' stamp: 'bar 10/5/2005 22:35'!
<= aECEntry 
	^ contents <= aECEntry contents! !

!ECEntry methodsFor: 'private' stamp: 'lr 3/29/2008 10:39'!
browseWith: anECContext 
	type := self guessTypeWith: anECContext.
	type
		ifNil: [^ false].
	SystemNavigation default
		browseClass: type.
	^ true! !

!ECEntry methodsFor: 'as yet unclassified' stamp: 'bar 8/14/2005 21:44'!
completion
	^ self contents asSymbol! !

!ECEntry methodsFor: 'accessing' stamp: 'bar 8/13/2005 18:56'!
contents
	^contents! !

!ECEntry methodsFor: 'as yet unclassified' stamp: 'bar 10/5/2005 22:36'!
contentsAsSymbol
	^ contents asSymbol ! !

!ECEntry methodsFor: 'detail information' stamp: 'lr 7/4/2009 10:42'!
createDescriptionWith: anECContext 
	| clazz |
	clazz := self guessTypeWith: anECContext.
	^ clazz 
		ifNil: [ ECEntryDescription label: self label ]
		ifNotNil: 
			[ ECEntryDescription 
				label: self label
				title: clazz printString
				description: clazz comment ]! !

!ECEntry methodsFor: 'as yet unclassified' stamp: 'lr 7/4/2009 10:42'!
descriptionWith: anECContext 
	description ifNotNil: [ ^ description ].
	^ description := self createDescriptionWith: anECContext! !

!ECEntry methodsFor: 'as yet unclassified' stamp: 'bar 3/1/2006 15:27'!
guessTypeWith: anECContext 
	^ nil! !

!ECEntry methodsFor: 'testing' stamp: 'bar 3/2/2006 17:50'!
isInstance
	^ false! !

!ECEntry methodsFor: 'testing' stamp: 'bar 3/2/2006 17:51'!
isLocal
	^ false! !

!ECEntry methodsFor: 'testing' stamp: 'bar 3/2/2006 17:51'!
isSelector
	^ false! !

!ECEntry methodsFor: 'accessing' stamp: 'bar 3/1/2006 15:40'!
label
	^ 'unknown'! !

!ECEntry methodsFor: 'printing' stamp: 'EduardoEscobar 1/10/2012 22:15'!
printOn: aStream 
	"Append to the argument, aStream, a sequence of characters that identifies the receiver."
	aStream nextPutAll: self class name;
		 nextPut: $(;
		 nextPutAll: contents;
		 nextPut: $,;
		 nextPutAll: type;
		 nextPut: $)! !

!ECEntry methodsFor: 'as yet unclassified' stamp: 'lr 7/4/2009 10:42'!
setContents: aString type: aSymbol 
	contents := aString.
	type := aSymbol! !

!ECEntry methodsFor: 'accessing' stamp: 'bar 8/13/2005 18:56'!
type
	^type! !

ECEntry subclass: #ECGlobalEntry
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECGlobalEntry methodsFor: 'as yet unclassified' stamp: 'bar 3/20/2006 15:20'!
guessTypeWith: anECContext 
	| globalEntry |
	globalEntry := Smalltalk at: contents ifAbsent: [^ nil].
	globalEntry isBehavior ifTrue: [^ globalEntry].
	globalEntry ifNotNil: [^ globalEntry class].
	^ nil! !

!ECGlobalEntry methodsFor: 'accessing' stamp: 'bar 3/1/2006 15:41'!
label
	^ 'global'! !

ECEntry subclass: #ECInstVarEntry
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECInstVarEntry methodsFor: 'as yet unclassified' stamp: 'bar 3/1/2006 15:27'!
guessTypeWith: anECContext 
	^ anECContext guessInstVarClass: contents! !

!ECInstVarEntry methodsFor: 'testing' stamp: 'EduardoEscobar 1/11/2012 00:13'!
isInstance
	^true! !

!ECInstVarEntry methodsFor: 'accessing' stamp: 'bar 3/1/2006 15:41'!
label
	^ 'instance variable'! !

ECEntry subclass: #ECLocalEntry
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECLocalEntry methodsFor: 'as yet unclassified' stamp: 'bar 3/1/2006 15:27'!
guessTypeWith: anECContext 
	^ (anECContext guessTempVarClass: contents type: type) 
		ifNil: [anECContext guessArgument: contents]! !

!ECLocalEntry methodsFor: 'testing' stamp: 'bar 3/2/2006 17:51'!
isLocal
	^true! !

!ECLocalEntry methodsFor: 'accessing' stamp: 'bar 3/1/2006 15:41'!
label
	^ 'local variable'! !

ECEntry subclass: #ECSelectorEntry
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECSelectorEntry methodsFor: 'private' stamp: 'damiencassou 1/6/2009 14:04'!
browseWith: anECContext
	^ self
		findMethodWith: anECContext
		do: [ :class :method | 
			Browser fullOnClass: class selector: method selector.
			true ]
		ifAbsent: [ :selector | 
			(SystemBrowser default name beginsWith: 'OB')
				ifTrue: [OBImplementorsBrowser openRoot: (OBSelectorNode on: selector).]
				ifFalse: [self systemNavigation browseAllImplementorsOf: selector].
				true ]! !

!ECSelectorEntry methodsFor: 'detail information' stamp: 'bar 3/8/2006 21:42'!
createDescriptionWith: anECContext 
	^ self
		findMethodWith: anECContext
		do: [:clazz :method | self
				methodSourceDescription: clazz
				method: method
				]
		ifAbsent: [:selector | self implementorsDescription: selector]! !

!ECSelectorEntry methodsFor: 'private' stamp: 'bar 3/20/2006 16:15'!
findMethodWith: anECContext do: foundBlock ifAbsent: notfoundBlock 
	| theClass result implementors |
	theClass := anECContext model theClass.
	result := theClass
				ifNil: [implementors := self systemNavigation allImplementorsOf: contents.
					implementors size == 1
						ifTrue: [| ref | 
							ref := implementors first.
							self lookupSelector: ref methodSymbol class: ref actualClass]
						ifFalse: [^ notfoundBlock value: contents]]
				ifNotNil: [self lookupSelector: contents class: theClass]. 
	^ foundBlock value: result first value: result second! !

!ECSelectorEntry methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
implementorsDescription: aSymbol 
	| implementors output |
	output := WriteStream on: String new.
	implementors := self systemNavigation allImplementorsOf: aSymbol.
	implementors isEmpty ifTrue: 
		[ ^ ECEntryDescription 
			label: 'symbol'
			title: '(no implementors)'
			description: 'This is just symbol.' ].
	implementors do: 
		[ :each | 
		output
			nextPutAll: each classSymbol printString;
			cr ].
	^ ECEntryDescription 
		label: self label
		title: '(Implementors)'
		description: output contents! !

!ECSelectorEntry methodsFor: 'accessing' stamp: 'bar 3/2/2006 21:52'!
label
	^ 'method'! !

!ECSelectorEntry methodsFor: 'private' stamp: 'bar 3/20/2006 15:38'!
lookupSelector: aSymbol class: aClass 
	"Look up the given selector in my methodDictionary.
	Return the corresponding method if found.
	Otherwise chase the superclass chain and try again.
	Return nil if no method is found."
	| lookupClass |
	lookupClass := aClass.
	[lookupClass isNil]
		whileFalse: [(lookupClass includesSelector: aSymbol)
				ifTrue: [^ Array
						with: lookupClass
						with: (lookupClass compiledMethodAt: aSymbol)].
			lookupClass := lookupClass superclass].
	^ nil! !

!ECSelectorEntry methodsFor: 'as yet unclassified' stamp: 'bar 3/30/2006 14:17'!
methodSourceDescription: aClass method: aCompiledMethod 
	| styler |
	styler := SHTextStylerST80 new.
	styler classOrMetaClass: aClass.
	^ ECEntryDescription
		label: self label
		title: aClass printString
		description: (styler styledTextFor: (self methodSourceOn: aCompiledMethod ) asText)! !

!ECSelectorEntry methodsFor: 'private' stamp: 'rjl 12/11/2008 21:07'!
methodSourceOn: aCompiledMethod 
	^ aCompiledMethod getSourceFor: aCompiledMethod selector
		in: aCompiledMethod methodClass! !

ECEntry subclass: #ECSelfEntry
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECSelfEntry methodsFor: 'as yet unclassified' stamp: 'bar 3/1/2006 15:27'!
guessTypeWith: anECContext 
	^ anECContext theClass! !

!ECSelfEntry methodsFor: 'accessing' stamp: 'bar 3/1/2006 15:42'!
label
	^ 'self'! !

ECEntry subclass: #ECSuperEntry
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECSuperEntry methodsFor: 'as yet unclassified' stamp: 'bar 3/1/2006 15:27'!
guessTypeWith: anECContext 
	^ anECContext theClass ifNotNil: [anECContext theClass superclass]! !

!ECSuperEntry methodsFor: 'accessing' stamp: 'bar 3/1/2006 15:42'!
label
	^ 'super'! !

Object subclass: #ECEntryDescription
	instanceVariableNames: 'title description label'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECEntryDescription class methodsFor: 'as yet unclassified' stamp: 'bar 3/2/2006 22:01'!
label: firstString 
	^ self new
		setLabel: firstString
		title: '(unknown)'
		description: nil! !

!ECEntryDescription class methodsFor: 'as yet unclassified' stamp: 'bar 3/1/2006 15:39'!
label: firstString title: secondString description: thirdString 
	^ self new 
		setLabel: firstString
		title: secondString
		description: thirdString! !

!ECEntryDescription methodsFor: 'accessing' stamp: 'bar 3/2/2006 22:02'!
description
(	description isNil or:[description isEmpty]) ifTrue:[^'-'].
	^ description! !

!ECEntryDescription methodsFor: 'accessing' stamp: 'bar 3/1/2006 15:39'!
label
	^ label! !

!ECEntryDescription methodsFor: 'initialize-release' stamp: 'lr 7/4/2009 10:42'!
setLabel: firstString title: secondString description: thirdString 
	label := firstString.
	title := secondString.
	description := thirdString! !

!ECEntryDescription methodsFor: 'accessing' stamp: 'bar 3/1/2006 15:20'!
title
^ title! !

Object subclass: #ECModel
	instanceVariableNames: 'clazz selectors narrowString entries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECModel commentStamp: '<historical>' prior: 0!
I'm an abstract class that stores the entries to be completed.!

!ECModel class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
class: aClass 
	| newInstance |
	newInstance := self basicNew initialize.
	newInstance setClass: aClass.
	^ newInstance! !

!ECModel methodsFor: 'accessing' stamp: 'ul 12/9/2010 23:05'!
addToEntries: anOrderedCollection

	| frontMatching innerMatching caseSensitive matchSubStrings |
	caseSensitive := ECPreferences caseSensitive.
	matchSubStrings := ECPreferences matchSubStrings.
	frontMatching := OrderedCollection new.
	innerMatching := OrderedCollection new.
	anOrderedCollection do: [ :each |
		| index |
		index := each contents 
			indexOfSubStringWithEmpty: narrowString
			caseSensitive: caseSensitive.
		index = 1
			ifTrue: [ frontMatching add: each ]
			ifFalse: [
				(matchSubStrings and: [ index > 1 ]) ifTrue: [
					innerMatching add: each ] ] ].
	innerMatching sort.
	frontMatching sort.
	entries
		 addAll: frontMatching;
		 addAll: innerMatching! !

!ECModel methodsFor: 'accessing' stamp: 'EduardoEscobar 1/10/2012 22:21'!
at: aNumber
	"Primitive. Assumes receiver is indexable. Answer the value of an indexable element in the receiver. Fail if the argument index is not an Integer or is out of bounds. Essential. See Object documentation whatIsAPrimitive."
	^ entries at: aNumber ! !

!ECModel methodsFor: 'action' stamp: 'kb 5/21/2011 22:50'!
completionAt: aNumber
	^ (self at: aNumber) completion separateKeywords, ' '! !

!ECModel methodsFor: 'accessing' stamp: 'bar 12/4/2004 17:20'!
entries
	^entries! !

!ECModel methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
entriesOfType: aSymbol 
	| collection |
	collection := entries select: [ :each | each type == aSymbol ].
	^ collection collect: [ :each | each contents ]! !

!ECModel methodsFor: 'accessing' stamp: 'bar 12/4/2004 17:21'!
entryCount
	^entries size! !

!ECModel methodsFor: 'accessing' stamp: 'bar 12/17/2004 22:45'!
hasMessage
	^ self message notNil! !

!ECModel methodsFor: 'initialize-release' stamp: 'EduardoEscobar 1/10/2012 22:22'!
initialize
	"Subclasses should redefine this method to perform initializations on instance creation"
	self reset! !

!ECModel methodsFor: 'accessing' stamp: 'bar 12/6/2004 14:18'!
initializeSelectors 
	self subclassResponsibility ! !

!ECModel methodsFor: 'testing' stamp: 'bar 3/3/2006 17:00'!
isEmpty
	^ entries isEmpty! !

!ECModel methodsFor: 'accessing' stamp: 'bar 1/6/2005 08:25'!
message
	^self isEmpty ifTrue: ['no completions found'] ifFalse: [nil]! !

!ECModel methodsFor: 'private' stamp: 'bar 12/4/2004 17:40'!
narrowString: aString 
	narrowString := aString! !

!ECModel methodsFor: 'action' stamp: 'bar 12/6/2004 11:53'!
narrowWith: aString
	self subclassResponsibility ! !

!ECModel methodsFor: 'testing' stamp: 'bar 12/4/2004 17:22'!
notEmpty 
	^self isEmpty not! !

!ECModel methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
reset
	self resetSelectors.
	self resetEntries.
	narrowString := String new! !

!ECModel methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
resetEntries
	entries := OrderedCollection new! !

!ECModel methodsFor: 'private' stamp: 'ul 9/26/2010 19:23'!
resetSelectors
	selectors
		ifNil: [ selectors := OrderedCollection new: 500 ]
		ifNotNil: [ selectors reset ]! !

!ECModel methodsFor: 'initialize-release' stamp: 'bar 1/6/2005 08:54'!
setClass: aClass 
	clazz := aClass.
	self initializeSelectors.
	self narrowWith: String new! !

!ECModel methodsFor: 'action' stamp: 'bar 3/1/2006 15:48'!
theClass
	^nil! !

!ECModel methodsFor: 'action' stamp: 'bar 12/6/2004 14:44'!
title
	^nil! !

!ECModel methodsFor: 'initialize-release' stamp: 'bar 12/5/2004 23:17'!
toggleExpand
	! !

ECModel subclass: #ECTypedModel
	instanceVariableNames: 'expanded'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECTypedModel commentStamp: '<historical>' prior: 0!
I'm the model for a typed completion, that means when a receiver class is known. I only have selectors. I have an expand toggle: When false I filter out Object and ProtoObject selectors. The default value is true.!

ECTypedModel subclass: #ECOverrideModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECOverrideModel commentStamp: '<historical>' prior: 0!
I'm used when completing a method override. I contain all selectors of the superclass minus the already implemented selectors of the current class. When a completion occurs I complete a method template with a send to super.!

!ECOverrideModel methodsFor: 'action' stamp: 'ul 10/29/2010 06:10'!
completionAt: aNumber 
	| output source declaration |
	source := self methodSourceAt: aNumber.
	declaration := (source lineCorrespondingToIndex: 1)
		withoutTrailingBlanks.
	output := WriteStream on: String new.
	output nextPutAll: declaration; cr; tab.
	(source includesSubString: 'subclassResponsibility') ifFalse: [
		(source includes: $^)
			ifTrue: [ output nextPut: $^ ].
		output
			nextPutAll: 'super ';
			nextPutAll: declaration ].
	^ output contents! !

!ECOverrideModel methodsFor: 'private' stamp: 'bar 12/6/2004 16:51'!
computeReturn: aString
	| lastLine |
	lastLine := aString lineCorrespondingToIndex: aString size.
	^(lastLine
				detect: [:each | each = $^]
				ifNone: [])
				ifNil: ['']
				ifNotNil: ['^'].
! !

!ECOverrideModel methodsFor: 'initialize-release' stamp: 'lr 7/4/2009 10:42'!
initializeSelectors
	self initializeSelectorsFor: clazz superclass.
	clazz methodDictionary keysDo: 
		[ :each | 
		| entry |
		entry := selectors 
			detect: [ :ea | ea contentsAsSymbol == each ]
			ifNone: [  ].
		entry notNil ifTrue: 
			[ selectors 
				remove: entry
				ifAbsent: [  ] ] ]! !

!ECOverrideModel methodsFor: 'private' stamp: 'lr 6/19/2010 12:49'!
methodAt: aNumber 
	^ clazz lookupSelector: (self at: aNumber) contentsAsSymbol! !

!ECOverrideModel methodsFor: 'private' stamp: 'ul 10/29/2010 06:02'!
methodSourceAt: aNumber 
	
	^(self methodAt: aNumber) getSource asString! !

!ECOverrideModel methodsFor: 'action' stamp: 'bar 12/6/2004 15:02'!
title
	^ '(override) ' , clazz superclass name! !

!ECTypedModel methodsFor: 'initialize-release' stamp: 'EduardoEscobar 1/10/2012 22:23'!
initialize
	"Subclasses should redefine this method to perform initializations on instance creation"
	super initialize.
	expanded := false.
! !

!ECTypedModel methodsFor: 'private' stamp: 'bar 12/6/2004 14:19'!
initializeSelectors
	self initializeSelectorsFor: clazz! !

!ECTypedModel methodsFor: 'private' stamp: 'lr 3/17/2010 19:50'!
initializeSelectorsFor: aClass 
	|excludedClasses|
	selectors reset.
	excludedClasses := (expanded ifTrue: [#()] ifFalse: [Object withAllSuperclasses]).
	selectors addAll: ((aClass allSelectorsForCompletionWithout: excludedClasses)
						collect: [:each | ECSelectorEntry contents: each type: #selector])! !

!ECTypedModel methodsFor: 'action' stamp: 'cmm 1/6/2007 21:49'!
narrowWith: aString 
	self
		narrowString: aString ;
		initializeSelectors.
	entries reset.
	self addToEntries: selectors! !

!ECTypedModel methodsFor: 'action' stamp: 'bar 3/1/2006 15:49'!
theClass
	^clazz! !

!ECTypedModel methodsFor: 'action' stamp: 'bar 12/6/2004 14:44'!
title
	^clazz name! !

!ECTypedModel methodsFor: 'action' stamp: 'lr 7/4/2009 10:42'!
toggleExpand
	expanded := expanded not.
	self initializeSelectors.
	self narrowWith: narrowString! !

ECModel subclass: #ECUntypedModel
	instanceVariableNames: 'instVars localVars includeVariables includeSelectors classVars'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECUntypedModel commentStamp: '<historical>' prior: 0!
When no receiver class is known, I'm the right model. I store all temporary variables, instance variables of the selected class and all selectors in system. For performance reasons I only collect selectors when at least one character is known.!

ECUntypedModel subclass: #ECUnseparatedModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECUnseparatedModel methodsFor: 'action' stamp: 'dr 10/30/2008 17:06'!
completionAt: aNumber 
	^ (self at: aNumber) completion! !

!ECUntypedModel class methodsFor: 'instance creation' stamp: 'bar 12/20/2004 09:02'!
class: aClass temporaries: aCollection 
	^self 
		class: aClass
		temporaries: aCollection
		additionals: #()
		variables: true
		selectors: true! !

!ECUntypedModel class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
class: aClass temporaries: aCollection additionals: additionalCollection variables: variablesBoolean selectors: selectorsBoolean 
	| newInstance |
	newInstance := self basicNew initialize.
	newInstance 
		setClass: aClass
		temporaries: aCollection
		additionals: additionalCollection
		variables: variablesBoolean
		selectors: selectorsBoolean.
	^ newInstance! !

!ECUntypedModel methodsFor: 'private' stamp: 'ul 10/29/2010 02:11'!
addAdditionals: aCollection 

	aCollection ifNil: [ ^self ].
	aCollection do: [ :each |
		each isLocal
			ifTrue: [ localVars add: each value ]
			ifFalse: [
				each isInstance
					ifTrue: [ instVars add: each value ]
					ifFalse: [ 
						each isSelector ifTrue: [ selectors add: each value ] ] ] ]! !

!ECUntypedModel methodsFor: 'private' stamp: 'ul 9/29/2010 19:01'!
addSelectors

	self
		initializeSelectors;
		addToEntries: selectors! !

!ECUntypedModel methodsFor: 'private' stamp: 'bar 10/5/2005 23:05'!
addVariables
	includeVariables
		ifFalse: [^ self].
	self addToEntries: localVars.
	self addToEntries: instVars.
	self addToEntries: classVars! !

!ECUntypedModel methodsFor: 'initialize-release' stamp: 'EduardoEscobar 1/10/2012 22:24'!
initialize
	"Subclasses should redefine this method to perform initializations on instance creation"
	super initialize.
	localVars := OrderedCollection new.
	instVars := OrderedCollection new.
	classVars := OrderedCollection new.
	includeSelectors := true.
	includeVariables := true! !

!ECUntypedModel methodsFor: 'initialize-release' stamp: 'ul 10/29/2010 06:04'!
initializeClassVars

	(clazz isNil or: [ includeVariables not ]) ifTrue: [ ^ self ].
	classVars := clazz theNonMetaClass allClassVarNames asOrderedCollection replace: [ :each | 
		ECClassVarEntry
			contents: each
			type: #classVar ]! !

!ECUntypedModel methodsFor: 'initialize-release' stamp: 'ul 9/27/2010 04:49'!
initializeInstVars
	(clazz isNil or: [ includeVariables not ]) ifTrue: [ ^ self ].
	instVars := clazz allInstVarNames asOrderedCollection replace: [ :each | 
		ECInstVarEntry 
			contents: each
			type: #instVar ].
	instVars add: (ECSelfEntry 
			contents: 'self'
			type: #self).
	instVars add: (ECSuperEntry 
			contents: 'super'
			type: #super)! !

!ECUntypedModel methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/16/2010 15:51'!
initializeSelectors
	self resetSelectors.
	includeSelectors ifFalse: 
		[ Smalltalk environment keysDo: 
			[ :each | 
			selectors add: (ECGlobalEntry 
					contents: each
					type: #globalVar) ].
		^ self ].
	narrowString ifEmpty: [ ^ self ].
	ECSymbols 
		contains: narrowString
		caseSensitive: ECPreferences caseSensitive
		do: 
			[ :each | 
			(includeVariables or: [ each first isLowercase ]) ifTrue: 
				[ selectors add: (ECSelectorEntry 
						contents: each
						type: #selector).
				selectors size > 500 ifTrue: [ ^ self ] ] ]! !

!ECUntypedModel methodsFor: 'accessing' stamp: 'cmm 1/8/2007 21:01'!
message
	^ (includeSelectors and: [ narrowString isEmpty ]) 
		ifTrue: 
			[ selectors size = 500 
				ifTrue: [ 'more...' ]
				ifFalse: [ 'press key for selectors' ] ]
		ifFalse: [ super message ]! !

!ECUntypedModel methodsFor: 'initialize-release' stamp: 'bar 4/11/2005 11:42'!
narrowString: aString 
	(narrowString isEmpty
			or: [aString isEmpty
			or: [aString first ~= narrowString first]])
		ifTrue: [self reset].
	super narrowString: aString! !

!ECUntypedModel methodsFor: 'action' stamp: 'bar 4/11/2005 11:35'!
narrowWith: aString 
	self narrowString: aString.
	self resetEntries.
	self addVariables.
	self addSelectors! !

!ECUntypedModel methodsFor: 'initialize-release' stamp: 'TestRunner 12/3/2009 13:24'!
noVariables
	includeVariables := false.! !

!ECUntypedModel methodsFor: 'initialize-release' stamp: 'lr 7/4/2009 10:42'!
setClass: aClass temporaries: aCollection additionals: additionalCollection variables: variablesBoolean selectors: selectorsBoolean 
	includeVariables := variablesBoolean.
	includeSelectors := selectorsBoolean.
	clazz := aClass.
	self initializeInstVars.
	self initializeClassVars.
	includeVariables 
		ifTrue: 
			[ localVars := aCollection.
			self addAdditionals: additionalCollection ]
		ifFalse: [ localVars := OrderedCollection new ].
	self
		resetEntries;
		addVariables;
		addSelectors! !

Object subclass: #ECTypeInfo
	instanceVariableNames: 'type kind temporaryOffset'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ocompletion-ECModel'!

!ECTypeInfo commentStamp: '<historical>' prior: 0!
I'm used in ECInstVarTypeGuesser to store found type informations.!

!ECTypeInfo class methodsFor: 'as yet unclassified' stamp: 'bar 12/2/2004 11:52'!
definedByLiteral: aClass 
	| newInstance |
	newInstance := self new.
	newInstance setType: aClass kind: 1.
	^ newInstance! !

!ECTypeInfo class methodsFor: 'as yet unclassified' stamp: 'bar 12/2/2004 11:52'!
definedByMessageSend: aClass 
	| newInstance |
	newInstance := self new.
	newInstance setType: aClass kind: 2.
	^ newInstance! !

!ECTypeInfo class methodsFor: 'as yet unclassified' stamp: 'bar 12/2/2004 12:02'!
definedByTemporaryVar: anInteger
	| newInstance |
	newInstance := self new.
	newInstance setType: nil kind: 3.
	newInstance temporaryOffset: anInteger.
	^ newInstance! !

!ECTypeInfo methodsFor: 'testing' stamp: 'bar 12/2/2004 11:47'!
isDefinedByMessageSend
	^kind == 2! !

!ECTypeInfo methodsFor: 'testing' stamp: 'bar 12/2/2004 11:47'!
isDefinedByTemporary
	^ kind == 3! !

!ECTypeInfo methodsFor: 'accessing' stamp: 'bar 12/2/2004 12:01'!
priority
	^kind! !

!ECTypeInfo methodsFor: 'as yet unclassified' stamp: 'bar 12/2/2004 12:01'!
setType: aClass kind: anInteger 
	type := aClass.
	kind := anInteger! !

!ECTypeInfo methodsFor: 'accessing' stamp: 'bar 12/2/2004 12:02'!
temporaryOffset
	^temporaryOffset! !

!ECTypeInfo methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
temporaryOffset: anInteger 
	temporaryOffset := anInteger! !

!ECTypeInfo methodsFor: 'accessing' stamp: 'bar 12/2/2004 11:44'!
type
	^type! !

!ECTypeInfo methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
type: aClass 
	type := aClass! !
